//***********************************************
//****************  G_Code.h  *******************
//***********************************************
//
//
//***********************************************
//****************  G_Code.h  *******************
//***********************************************
#include "LCD.h"		//LCD functions for use in this
#include "Stepper.h"	


//List of valid codes and function description.
/*
RS274NGC G-CODE PROGRAMMING 

G and M Code Modal Groups
group 1 = {G0, G1, G2, G3, G80, G81, G82, G83, G84, G85, G86, G87, G88, G89} - motion  
group 2 = {G17, G18, G19} - plane selection  
group 3 = {G90, G91} - distance mode  
group 5 = {G93, G94} - spindle speed mode  
group 6 = {G20, G21} - units  
group 7 = {G40, G41, G42} - cutter diameter compensation  
group 8 = {G43, G49} - tool length offset  
group 10 = {G98, G99} - return mode in canned cycles  
group12 = {G54, G55, G56, G57, G58, G59, G59.1, G59.2, G59.3} coordinate system selection

group 2 = {M26, M27} - axis clamping  
group 4 = {M0, M1, M2, M30, M60} - stopping  
group 6 = {M6} - tool change  
group 7 = {M3, M4, M5} - spindle turning  
group 8 = {M7, M8, M9} - coolant  
group 9 = {M48, M49} - feed and speed override bypass


Words acceptable to the interpreter
D	Tool radius compensation number
F	Feedrate
G	General function see list below
H	Tool length offset
I	X-axis offset for arcs
J	Y-axis offset for arcs
K	Z-axis offset for arcs
L	Number of repetitions in canned cycles
M	Miscellanious function
N	Line Number
P	Dwell time in G4 and canned cycles
Q	Feed increment in G83 canned cycle
	Key used with G10
R	arc radius
S	canned cycle plane
T	Tool selection
X	X-axis of machine
Y	Y-axis of machine
Z	Z-axis of machine
*/

/*
G00	rapid positioning  
G01	linear interpolation  
G02	circular/helical interpolation (clockwise)  
G03	circular/helical interpolation (c-clockwise)  
G04	dwell  
G10 coordinate system origin setting  
G17 xy plane selection  
G18 xz plane selection  
G19 yz plane selection  
G20 inch system selection  
G21 millimeter system selection  
G40 cancel cutter diameter compensation  
G41 start cutter diameter compensation left  
G42 start cutter diameter compensation right  
G43 tool length offset (plus)  
G49 cancel tool length offset  
G53 motion in machine coordinate system  
G54 use preset work coordinate system 1  
G55 use preset work coordinate system 2  
G56 use preset work coordinate system 3  
G57 use preset work coordinate system 4
G58 use preset work coordinate system 5  
G59 use preset work coordinate system 6  
G59.1 use preset work coordinate system 7  
G59.2 use preset work coordinate system 8  
G59.3 use preset work coordinate system 9  
G80 cancel motion mode (includes canned)  
G81 drilling canned cycle  
G82 drilling with dwell canned cycle  
G83 chip-breaking drilling canned cycle  
G84 right hand tapping canned cycle  
G85 boring, no dwell, feed out canned cycle  
G86 boring, spindle stop, rapid out canned  
G87 back boring canned cycle  
G88 boring, spindle stop, manual out canned  
G89 boring, dwell, feed out canned cycle  
G90 absolute distance mode  
G91 incremental distance mode  
G92 offset coordinate systems  
G92.2 cancel offset coordinate systems  
G93 inverse time feed mode  
G94 feed per minute mode  
G98 initial level return in canned cycles

M0 program stop  
M1 optional program stop  
M2 program end  
M3 turn spindle clockwise  
M4 turn spindle counterclockwise  
M5 stop spindle turning  
M6 tool change  
M7 mist coolant on
M8 flood coolant on  
M9 mist and flood coolant off  
M26 enable automatic b-axis clamping  
M27 disable automatic b-axis clamping  
M30 program end, pallet shuttle, and reset  
M48 enable speed and feed overrides  
M49 disable speed and feed overrides  
M60 pallet shuttle and program stop

*/

#define FastFeed				100	//100 fast 
#define SlowFeed				10	//10  slow
#define StopSpindle				5	//M5 stop spindle turning 
#define SpindleRotate			3	//M3 turn spindle clockwise


/*****************************************************************************/
/*                                 Functions                                */
/*****************************************************************************/

//External Functions
void G_CodeRun(char GC_L, int iGC_N, char GC_1L, float fGC_1, char GC_2L,
             float fGC_2, char GC_3L, float fGC_3, char GC_4L,float fGC_4);

void G_Code_G0(void);
void G_Code_G1(void);
void G_Code_G2(void);
void G_Code_G3(void);
void G_Code_G4(void);
void G_Code_G10(void);

/*****************************************************************************/

//Internal Functions
//None
