//***********************************************
//******************Manual.c*********************
//***********************************************
//
//
//***********************************************
//******************Manual.c*********************
//***********************************************
#include "Manual.h"

//Module Globals
long X_mm = 0;
long Y_mm = 0;
long Z_mm = 0;
long X_AtoD = 0;
long Y_AtoD = 0;
long Z_AtoD = 0;

signed long X_FeedRate = 0;
signed long Y_FeedRate = 0;
signed long Z_FeedRate = 0;

int zFeed = 0;
int dir;

//Functions
/******************************************************************/
void Manual_initADC(int amask)	{

/*
**********************************
AD1PCFG - A to D function Register
**********************************
bit 15-13 Unimplemented: Read as 0
bit 12-0 PCFG12:PCFG0: Analog Input Pin Configuration Control bits(1)
		1 = Configured in Digital mode; I/O port read enabled
		0 = Configured in Analog mode; I/O port read disabled, A/D samples pin voltage
*/
	AD1PCFG = amask;		//select anaolog input pins
/*
**********************************
AD1CON1 - A to D function register
**********************************
bit 15 ADON: A/D Operating Mode bit
			1 = A/D Converter module is operating
			0 = A/D Converter is off
bit 14 Unimplemented: Read as 0
bit 13 ADSIDL: Stop in Idle Mode bit
			1 = Discontinue module operation when device enters Idle mode
			0 = Continue module operation in Idle mode
bit 12-10 Unimplemented: Read as 0
bit 9-8 FORM1:FORM0: Data Output Format bits
			11 = Signed fractional (sddd dddd dd00 0000)
			10 = Fractional (dddd dddd dd00 0000)
			01 = Signed integer (ssss sssd dddd dddd)
			00 = Integer (0000 00dd dddd dddd)
bit 7-5 SSRC2:SSRC0: Conversion Trigger Source Select bits
			111 = Internal counter ends sampling and starts conversion (auto-convert)
			110 = Reserved
			10x = Reserved
			011 = Reserved
			010 = Timer3 compare ends sampling and starts conversion
			001 = Active transition on INT0 pin ends sampling and starts conversion
			000 = Clearing SAMP bit ends sampling and starts conversion
bit 4-3 Unimplemented: Read as 0
bit 2 ASAM: A/D Sample Auto-Start bit
			1 = Sampling begins immediately after last conversion completes. SAMP bit is auto-set.
			0 = Sampling begins when SAMP bit is set
bit 1 SAMP: A/D Sample Enable bit
			1 = A/D sample/hold amplifier is sampling input
			0 = A/D sample/hold amplifier is holding
bit 0 DONE: A/D Conversion Status bit
			1 = A/D conversion is done
			0 = A/D conversion is NOT done
*/
	// Bit    15 14 13 12  11 10 9 8     7 6 5 4  3 2 1 0      
	//0b   00=0  0  0  0   0  0  0 0  E0=1 1 1 0  0 0 0 0
	AD1CON1 = 0x00E0;		//Auto conversion sequence start after sampling

/*
**********************************
AD1CON2 - A to D function register
**********************************
bit 15-13 VCFG2:VCFG0: Voltage Reference Configuration bits
					VR+		VR-
			000 = AVdd		AVss
			001
			010
			011
			1xx
bit 12-11 Unimplemented: Read as 0
bit 10 CSCNA: Scan Input Selections for CH0+ S/H Input for MUX A Input Multiplexer Setting bit
			1 = Scan inputs
			0 = Do not scan inputs
bit 9-8 Unimplemented: Read as 0
bit 7 BUFS: Buffer Fill Status bit (valid only when BUFM = 1)
			1 = A/D is currently filling buffer 08-0F, user should access data in 00-07
			0 = A/D is currently filling buffer 00-07, user should access data in 08-0F
bit 6 Unimplemented: Read as 0
bit 5-2 SMPI3:SMPI0: Sample/Convert Sequences Per Interrupt Selection bits
			1111 = Interrupts at the completion of conversion for each 16th sample/convert sequence
			1110 = Interrupts at the completion of conversion for each 15th sample/convert sequence
			0001 = Interrupts at the completion of conversion for each 2nd sample/convert sequence
			0000 = Interrupts at the completion of conversion for each sample/convert sequence
bit 1 BUFM: Buffer Mode Select bit
			1 = Buffer configured as two 8-word buffers (AD1BUFn<15:8> and AD1BUFn<7:0>)
			0 = Buffer configured as one 16-word buffer (AD1BUFn<15:0>)
bit 0 ALTS: Alternate Input Sample Mode Select bit
			1 = Uses MUX A input multiplexer settings for first sample, then alternates between MUX B and
			MUX A input multiplexer settings for all subsequent samples
			0 = Always uses MUX A input multiplexer settings
*/
	// Bit    15 14 13 12  11 10 9 8     7 6 5 4  3 2 1 0      
	//0b   00=0  0  0  0   0  0  0 0  00=0 0 0 0  0 0 0 0
	AD1CON2 = 0;			//use MUXA, AVss and AVdd are used for VRef+/-
/*
**********************************
AD1CON3 - A to D function register
**********************************
bit 15 ADRC: A/D Conversion Clock Source bit
1 = A/D internal RC clock
0 = Clock derived from system clock
bit 14-13 Unimplemented: Read as 0
bit 12-8 SAMC4:SAMC0: Auto-Sample Time bits
11111 = 31 TAD

00001 = 1 TAD
00000 = 0 TAD (not recommended)
bit 7-0 ADCS7:ADCS0: A/D Conversion Clock Select bits
11111111 = 128  TCY
00000001 = TCY
00000000 = TCY/2
*/
	// Bit    15 14 13 12  11 10 9 8     7 6 5 4  3 2 1 0      
	//0b   1F=0  0  0  0   0  0  0 1  02=0 0 0 0  0 0 1 0
	AD1CON3 = 0x0102;		//Tad = 31 x Tad Tad=125ns

/*
**********************************
AD1CSSL - A to D function register
**********************************
bit 15-13 Unimplemented: Read as 0
bit 12-0 CSSL12:CSSL0: A/D Input Pin Scan Selection bits(
1 = Corresponding analog channel selected for input
0 = Analog channel omitted from input scan
*/
	AD1CSSL = 0;			//no scanning required

//Once set up, turn on A to D
	AD1CON1bits.ADON = 1;	//turn on ADC

}//initADC

/******************************************************************/
void Manual_Mode(void){

	
	//homeLCD();	
   	//putsLCD( "MM   Z= ");
	Z_AtoD = readADC(z_channel);
	//printf("%04lu",Z_AtoD);
	//setLCDC(0x40);      // set the cursor Line2
    //putsLCD( "X= ");
    X_AtoD = readADC(x_channel);
    //printf("%04lu",X_AtoD);
	//setLCDC(0x49);      // set the cursor Line2
    //putsLCD( "Y= ");
    Y_AtoD = readADC(y_channel);
	//printf("%04lu",Y_AtoD);

	CalcFeedRates();

}//ManualMode

/******************************************************************/	

int readADC(int ch)	{

	AD1CHS = ch;			//select analog input

	AD1CON1bits.SAMP = 1;	//start sampling
	while (!AD1CON1bits.DONE);	//wait for conversion to complete
    
	AD1CON1bits.SAMP = 1;	//start sampling
	while (!AD1CON1bits.DONE);	//wait for conversion to complete
    

	return ADC1BUF0;			//read the conversion result

}//readADC
/******************************************************************/
void CalcFeedRates(void)	{

//int dir;
int i=0;
int Z_Active = 0;
long xFeed, yFeed=0 ;

	Stepper_InitIO();
// max is 1024,  min is 0.
// max Feedrate would be 1024/2 = 512

 	X_FeedRate = X_AtoD - 512;
	Y_FeedRate = Y_AtoD - 512;
	Z_FeedRate = Z_AtoD;

	if ((!BtnZ_Down)|(!BtnZ_Up)){
		Z_Active = 1;
	}

if (Z_Active == 0)	{

	//move x
	if (X_FeedRate > 0){dir=1;} //set direction
	else 
		dir=0;

	X_FeedRate = abs(X_FeedRate);
	X_FeedRate = 512 - X_FeedRate; //stick moved from centre
	//large number that reduces as the stick is moved from ctr.

	if (X_FeedRate > 375)	{
		xFeed = 10000;	
	}
	else
		xFeed = (long) X_FeedRate * 16;

	//if (X_FeedRate < 200)	{	//limit max speed
	//	(xFeed=200);
	//}

	//while (_T3IE == 1);
	if (_T3IE == 0)	{ //if the interrupt is not busy
		if (X_FeedRate < 450) { //Null band
			SingleStepX(xFeed, dir);
		}
	}

	//move y
	if (Y_FeedRate >0){dir=0;} //note reversed due to joystick wireing
	else 
		dir=1;

	Y_FeedRate = abs(Y_FeedRate);
	Y_FeedRate = 512 - Y_FeedRate;

	if (Y_FeedRate > 375) {
		yFeed = 10000;
		
	}
	else
		yFeed = (long) Y_FeedRate * 16;

	//if (Y_FeedRate < 200)	{
	//	(yFeed=200);
	//}

	//while (_T4IE == 1);
	if (_T4IE == 0)	{
		if (Y_FeedRate < 450) { //Null band
			SingleStepY(yFeed, dir);
		}
	}		

		if ((X_FeedRate > 450)&(Y_FeedRate >450)){
			setLCDC(0x40);      // set the cursor Line2
    		putsLCD( "X= ");
    		printf("%04lu",X_AtoD);
			setLCDC(0x49);      // set the cursor Line2
    		putsLCD( "Y= ");
			printf("%04lu",Y_AtoD);
		}
}//end if Z_Active

if (Z_Active == 1)	{

  	//move z
	// set Z drection according to btn pushed
	if (BtnZ_Down)	{
		dir = 0;
	}
	if (BtnZ_Up)	{
		dir = 1;
	}

	Z_FeedRate = abs(Z_FeedRate);
		
	zFeed = (long) Z_FeedRate * 32;

	if (zFeed < 900){
		zFeed = 900;
	}
	
//	while (_T5IE == 1);
	for (i=0; i<=15; ++i){
		while (_T5IE == 1);
		SingleStepZ(zFeed, dir);
	}
	Z_Active = 0;

}//end if Z Active

}
