/***********************************************************
************************PIC_CNC*****************************
************************************************************
*
*Notes for G_Code syntax for input file.
*The Progamme has been developed using ACE DXF Converter
*and Eagle PCB.
*Performance with other converters or G_Code from other
* sources should be checked before running.
*
*Basic File requirement.
*	1) Max line length is 50 characters long.
*	2) Alows Blank lines
*	3) One instruction per line
*	4) F codes should be added to the file before the GCode
*		requires it	and should be the only instruction on
*		the line.
*
*		Warning!
*		G0 feed fast feed
*		G1 - If no F code is found the default slowest feed
*		rate will be used
*	5) M Codes should be added to the file as required.
*		This should be the only instruction on the line.
*		If more than one Mcode is required, each should be
*		placed on a new line
*		Default for Rapid positioning is StopSpindle
*		Default for Linear interpolation is RotateSpindle
*	6) The default input filename is GCode.txt and should
*		reside in the SDcard(Alternate file name in code below
*	7) Any line started with a open bracket is ignored
*		as comment
*	8) Works on GCode with a space between axis data
************************************************************
************************PIC_CNC*****************************
************************************************************/

#include <p24fj128ga010.h>
_CONFIG1( JTAGEN_OFF & GCP_OFF & GWRP_OFF & ICS_PGx2 & FWDTEN_OFF)
_CONFIG2(  FCKSM_CSDCMD & OSCIOFNC_OFF & POSCMOD_HS)

#include "PIC_CNC.h"

/***********************************************************
 * Function:        int main(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Overview:        
 *
 * Note:           Modules:Stepper, Manual, G_Code External
 * functions are prefixed with the module name, internal functions 
 * have no prefix. This does not apply to third party modules like 
 * SPI, FSIO and LCD.
 *  
 ************************************************************/

//File type switch
int BinFile  = 1; //1 to run binary file, 0 if a standard G Code 
//Buffer size switch 
int Buff_512 = 1; //1 to use large file buffer(is set to 1024 now)

//*************************************************************
int main( void )    {

SetupSDosc();//secondary oscillator for the sd card interface
Setup(); //main function is inits and to turn on the

//do{	
			
	OpenReadSDFile(); //sets the file name to open + has switches for more
					//SD File utilities, card write, search, multiple file open etc. 


	//setLCDC(0x40);      // set the cursor Line2
	//To indicate the file type being run
	//if (BinFile == 1)	{
	//	putsLCD( "Binary File     ");
	//}
	//else
	//	putsLCD( "G Code File     ");

	while(BtnTA == 1); //wait for key down

	do	{
		if (BtnTA == 0)	{ //Joystick top left key down
			clrLCD();	
			putsLCD( "Manual Mode     ");
			while(BtnTA == 0);  //wait for key up
		
			do	{
				Manual_Mode(); //Loop in manual mode to update any new ADC value
			}
			while (BtnTA == 1);//remain in manual until next key down

			homeLCD();
			putsLCD( "Auto Mode       ");
			while(BtnTA == 0);  //wait for key up
			setLCDC(0x40);      // set the cursor Line2
   			putsLCD( "Running....     ");	
		}

		if (BinFile == 1)	{
			if (ItemsRead == 0) { //ItemsRead is the number of GCode lines read in
				ReadBinData();	  //the value is decremented as code lines are actioned
			}
		 	if (Buff_512)	{
				LoadValues512();//Loads 512 or 1024 Bytes (32 or 64 G-Code instructions)
			}
			else
				LoadValues16(); //loads 16Bytes (a single G_Code instruction from Bin file)

			Stepper_InitIO();   //Init port A and the pulse delay timer
			Stepper_Run3DLine(); 
			Stepper_SaveValues();
		}

		if (BinFile == 0)	{
		
	 		do	{
				ReadData();	//Reads a line of text from the file
			}while (LineBuffer[1] == 40);//skip comment lines
		
			GetCodes();	//Words from the line put into globals

			//Control now passed to the G_Code module
			G_CodeRun(Inst_Code_Letter, iInst_Code, Dim_1_Letter, fDim_1, Dim_2_Letter,
				fDim_2, Dim_3_Letter, fDim_3, Dim_4_Letter,fDim_4);
			
			FlushBuffer(); //Flush buffer ready for new data
		}
	}
	while ((EOF_Flag !=1)|(ItemsRead != 0)); //loop to end of file

	homeLCD();
   	putsLCD( "Program  ");
	setLCDC(0x40);      // set the cursor Line2
   	putsLCD( "End             ");

	FSfclose (pointer); //close the file

//	while(BtnTA == 1); //loop while key up

//}
//while(BtnTA == 0); //loop while key down
		
 while(1); 
} // main
//************************************************************************

void SetupSDosc(void)
{
 // Turn on the secondary oscillator part of sd card interface
   __asm__ ("MOV #OSCCON,w1");
   __asm__ ("MOV.b #0x02, w0");
   __asm__ ("MOV #0x46, w2");
   __asm__ ("MOV #0x57, w3");
   __asm__ ("MOV.b w2, [w1]");
   __asm__ ("MOV.b w3, [w1]");
   __asm__ ("MOV.b w0, [w1]");

   // Activate the RTCC module  part of sd card interface
   __asm__ ("mov #NVMKEY,W0");
   __asm__ ("mov #0x55,W1");
   __asm__ ("mov #0xAA,W2");
   __asm__ ("mov W1,[W0]");
   __asm__ ("nop");
   __asm__ ("mov W2,[W0]");
   __asm__ ("bset RCFGCAL,#13");
   __asm__ ("nop");
   __asm__ ("nop");
   RCFGCALbits.RTCPTR0 = 1;
   RCFGCALbits.RTCPTR1 = 1;

   // Set it to the correct time
   // These values won't be accurate
   RTCVAL = 0x0007;   
   RTCVAL = 0x0717;
   RTCVAL = 0x0208;
   RTCVAL = 0x2137;
   RCFGCAL = 0x8000;

}//SetupSDosc
/************************************************************/
void Setup(void)	{

	//Initializations
	Stepper_InitIO(); 
    initLCD();
    
    // Start Message    
    putsLCD( "****PIC__CNC****");
	setLCDC(0x40);      // set the cursor Line2
	putsLCD( "Run G-Code");
	Manual_initADC( AINPUTS);

	//Initialize the SD/MMC library
   	//while (!FSInit());
}
/************************************************************/

void OpenReadSDFile(void)	{

	//This block required for additional file procedures
	//see FSConfig.h
   	//char path[30];
   	//char count = 30;
    //char * pointer2;
  	//unsigned char size = 0, i;
	
	if (EOF_Flag == 1){ //It may be after restart
		EOF_Flag = 0; 
		FilePointer = 0;
		ItemsRead = 0;
		PosInFile = 0;
		LineNumber = 0;
	}

 	//Initialize the SD/MMC library
   	while (!FSInit());
	
	GetFileToOpen();	//select file to open and save as global

/*
   	//Open file 1 in read mode
   	//next line to contain the file name to be used
	if (BinFile == 1)	{
		//pointer = FSfopen ("StepBin.txt", "r");
		pointer = FSfopen ("PIC24Bin.txt", "r");
	}
	else{
   		pointer = FSfopen ( SaveFileName, "r");
		//pointer = FSfopen ("GCodeT.txt", "r");	
	}
*/
	pointer = FSfopen ( SaveFileName, "r");

   	FilePointer = (int)pointer; //save file pointer as global   
   	clrLCD();
   
   	if ((pointer == NULL)|(pointer == 0x0000)){
		homeLCD();	
   		putsLCD( "File Error!");
		setLCDC(0x40);      // set the cursor Line2
   		putsLCD( "File not found");
		while(1);
   	}  	//file open error
	
   	clrLCD();
   	homeLCD();	
   	putsLCD( "File Open");
   	setLCDC(0x40);      // set the cursor Line2
   	//putsLCD("GCode.txt");
	putsLCD(SaveFileName);

}//Open&ReadSDFile
/************************************************************/
void GetFileToOpen(void)	{
//Searches the SD card root directory for :
//Flies with File attribute set to archive
//Files with .TXT file extension

SearchRec rec;
unsigned char attributes = ATTR_ARCHIVE;
char name[] = "*.txt";
int result = 1;

do{
	//Test SD card for valid file type
	result= FindFirst(name,attributes,&rec);
	if (result==0){//success
		clrLCD();	
   		putsLCD(rec.filename);
		setLCDC(0x40);      // set the cursor Line2
   		putsLCD( "NextTA SelectTB");
	}
	if (result!=0)	{//failure
		clrLCD();	
   		putsLCD("'FindFist' error");
		setLCDC(0x40);      // set the cursor Line2
   		putsLCD( "*.TXT Failed    ");
		while(1);
	}

	do{
		if (result ==0)	{//can only "FindNext" if "FindFirst" is success 
		//loop on find next through all files
			if (BtnTA ==0){//BtnTA has been pressed for next file
				while (BtnTA==0);//wait for key up

				result= FindNext(&rec);//check to see if there is another file
				if (result==0){//success
					homeLCD();//
					putsLCD( "                ");
					homeLCD();
					putsLCD(rec.filename);
				}
				//if (result!=0){//failure
				//No more files so go back to first
				//}	
			}
		}
	}
	while ((BtnTB == 1)&(result==0)); // while Btn press
	//while (BtnTB == 0);//wait for key up
}
while (BtnTB == 1); // while Btn press

while (BtnTB == 0);//wait for key up then exit

//save the selected filename to global
strcpy(SaveFileName, rec.filename);

}//GetFileToOpen
/************************************************************/
void ReadBinData(void)	{
int CharNum=0;

  	//Read into receivebuffer, Num.of bytes, Num.of Items, File handle
  	//FSfread returns the number of bytes returned 
 
  	EOF_Flag=0;
  	pointer = FilePointer; 	//set the file pointer
  	LineNumber += 1;		//move on line counter

 	//pointer is the file, pos in file, zero ofset from file beginning
  	if(FSfseek(pointer,PosInFile,0) !=0) { //set file pos to last char & test
		homeLCD();	
 		putsLCD( "File Error!");
		setLCDC(0x40);      // set the cursor Line2
   		putsLCD( "Invalid seek pt");
		EOF_Flag=1;
  	}
  
  	if (EOF_Flag == 0) {
  		
		if (Buff_512)	{
			ItemsRead = FSfread (BlockBuffer, 16, 64, pointer);
			//32 for 512 buffer, 64 for 1024
			BuffPoint = 0; //reset buffer pointer
		}
		else //{	//do 16 bit read
  			ItemsRead = FSfread (LineBuffer, 16, 1, pointer);
		//	ItemsRead = 0; //to read line by line
		//}//else	
  	}//end if

	if (FSfeof(pointer)) {
 		// its the end of the file
		// if ItemsRead >0 there is data in the buffer to finish off before
		// ending the prog.
		
		EOF_Flag = 1; //Flag EOF
		FSfclose (pointer); //close the file
	}//if
 	else
		//not yet at the end of file so save position
    	PosInFile = FSftell(pointer);


}//ReadBinLine
/************************************************************/
void LoadValues512(void) {
//takes data from a 512 data buffer and serves up line by line.
//Record structure
/*	
		BinArray(0) = AscW(Inst_Code_Letter)
        BinArray(1) = cInst_Code
		X Data
        sInt32_to_Hbytes(iXdim_1)
        BinArray(2) = lsb 'short lsb
        BinArray(3) = msb 'short msb

        Y Data
        sInt32_to_Hbytes(iYdim_2)
        BinArray(4) = lsb 'short lsb
        BinArray(5) = msb 'short msb

        Z Data
        sInt32_to_Hbytes(iZdim_3)
        BinArray(6) = lsb 'short lsb
        BinArray(7) = msb 'short msb

        I Data
        sInt32_to_Hbytes(iIdim_4)
        BinArray(8) = lsb 'short lsb
        BinArray(9) = msb 'short msb

        J Data
        sInt32_to_Hbytes(iJdim_5)
        BinArray(10) = lsb 'short lsb
        BinArray(11) = msb 'short msb

        BinArray(12) = FP_Char
        sInt32_to_Hbytes(iFP_Num)
        BinArray(13) = lsb  'short lsb

        BinArray(14) = bActiveFlags

        BinArray(15) = AscW("#")End of record filler
*/

unsigned int msb0, lsb0 = 0;
	
	if (ItemsRead != 0)	{ //Buffer 512 bytes long 16x16bytes

		if (BlockBuffer[BuffPoint+0]==0x47) {// Its a "G" code

			if (BlockBuffer[BuffPoint+1]==0x00) {// Its a "G0" code 
				Feed_Rate = 10;	//was 5 Fast 
			}
		
			if (BlockBuffer[BuffPoint+1]==0x01) {// Its a "G1" code
					if (BlockBuffer[BuffPoint+12]==0x46) {// & there is feed rate data
						Feed_Rate = BlockBuffer[BuffPoint+13]; // set to value
					}
					if (Feed_Rate == 0) Feed_Rate = 25;	//Slow G1 with no feed rate 
					//*******************************************************
					Feed_Rate = 15; //Hard code Feed Rate to override the file
					//*******************************************************
			}										
		}

		//X Data is buffer 3, 2
		msb0= BlockBuffer[BuffPoint+3];
		lsb0= BlockBuffer[BuffPoint+2];
		lX_Steps = (msb0<<8)+lsb0;
	
		//Y Data is buffer 5, 4
		msb0= BlockBuffer[BuffPoint+5];
		lsb0= BlockBuffer[BuffPoint+4];
		lY_Steps = (msb0<<8)+lsb0;

		//Z Data is buffer 7, 6
		msb0= BlockBuffer[BuffPoint+7];
		lsb0= BlockBuffer[BuffPoint+6];
		lZ_Steps = (msb0<<8)+lsb0;
	
/*		Set the Active axis
		This data is included in the Binary string to indicate any axis containing
		new data. G Code G1 X=5 Y=5
		Z is not mentioned therefore the variable for z will be set to zreo
		The active flag would only be set to show X&Y as changing
		If Z is not "active" the value of Z is made equal to the saved value
		of Z to give dletaZ = 0 ie no Z movement.
		If the Z axis was set to active, Z would move to zero
		
		Active Flag byte
		bit7 X, bit6 Y, bit5 Z, bit4 I
		0000 = 00 all off
		0010 = 20 Z move only
		1100 = C0 X&Y move

		TO DO here, may need to include:
		X only
		Y only
		X&Z
		Y&Z
		XYZ
*/
		switch (BlockBuffer[BuffPoint+14])	{
			case 0x00:
				GX_On = 0;
				GY_On = 0;
				GZ_On = 0;
				break;
			case 0x20:
				GX_On = 0;
				GY_On = 0;
				GZ_On = 1;
				break;
			case 0xC0:
				GX_On = 1;
				GY_On = 1;
				GZ_On = 0;
				break;
			default:
				GX_On = 0;
				GY_On = 0;
				GZ_On = 0;
		}//switch
		--ItemsRead;	 //decrement Items Read, when value is 0, read next line
		BuffPoint += 16; //move buffer pointer to next G_Code instruction.

	}//if(ItemsRead != 0)

}//LoadValues512
/************************************************************/
void LoadValues16(void)	{

unsigned int msb0,lsb0 = 0;

	//X Data is linebuffer 3, 2
	msb0= LineBuffer[3];
	lsb0= LineBuffer[2];
	lX_Steps = (msb0<<8)+lsb0;
	
	//Y Data is linebuffer 5, 4
	msb0= LineBuffer[5];
	lsb0= LineBuffer[4];
	lY_Steps = (msb0<<8)+lsb0;

	//Z Data is linebuffer 7, 6
	msb0= LineBuffer[7];
	lsb0= LineBuffer[6];
	lZ_Steps = (msb0<<8)+lsb0;
	
	//Set the Active axis
	//Active Flag byte
	//bit7 X, bit6 Y, bit5 Z, bit4 I
	//0000 = 00 all off
	//0010 = 20 Z only
	//1100 = C0 X&Y
	switch (LineBuffer[14])	{
		case 0x00:
			GX_On = 0;
			GY_On = 0;
			GZ_On = 0;
			break;
		case 0x20:
			GX_On = 0;
			GY_On = 0;
			GZ_On = 1;
			break;
		case 0xC0:
			GX_On = 1;
			GY_On = 1;
			GZ_On = 0;
			break;
		default:
			GX_On = 0;
			GY_On = 0;
			GZ_On = 0;
	}//switch

	--ItemsRead;	//decrement Items Read, when 0 read next line

}//LoadValues16
/************************************************************/
void ReadData(void)	{

int CharNum=1;

  	//Read into buffer, Num.of bytes, Num.of Items, File handle
  	//FSfread returns the number of items read from the file 
 
  	EOF_Flag=0;
  	pointer = FilePointer; 	//set the file pointer
  	LineNumber += 1;		//move on line counter
	

	//pointer is the file, pos in file, zero ofset from file beginning
	if(FSfseek(pointer,PosInFile,0) !=0) { //set file pos to last char & test
		homeLCD();	
 		putsLCD( "File Error!");
		setLCDC(0x40);      // set the cursor Line2
   		putsLCD( "Invalid seek pt");
		EOF_Flag=1;
  	}
  	
  	if (EOF_Flag == 0) {
  		do {
  			ItemsRead = FSfread (CharBuffer, 1, 1, pointer);
			LineBuffer[CharNum]=CharBuffer[0];
			CharNum++;

			if (CharBuffer[0]== 40)	{ //Open bracket - GCode comment line
				//loop through and discard the data.
				do	{
					ItemsRead = FSfread (CharBuffer, 1, 1, pointer);
					// Check if this is the end of the file
    				if (FSfeof (pointer)) {
						break;
    				}
				//Quickly read line checking for line feed
				}while (CharBuffer[0] !=10);
			}

 			// Check if this is the end of the file
    		if (FSfeof (pointer)) {
				FSfclose (pointer);
				EOF_Flag=1;
				break;
    		}

		}while (CharBuffer[0] !=10); //while charbuffer is not line feed char.

    	PosInFile=FSftell(pointer); //where we are after reading line

	//Note:
	//Inline reporting using the LCD functions slows things down to much here
	//uncomment for debug only.
	//Report line number in text file to LCD.
		setLCDC(0x0b);//pos 11
		printf("%lu",LineNumber);

		if (LineNumber==691){
			//Debug trap
			setLCDC(0x40);      // set the cursor Line2
		}

  	}//end if

}//ReadLine

/********************************************************/
void GetCodes(void)	{
//Takes a line of code and splits out the G-code words.
//Note code looks for a space between words, edit if Gcode
//runs words together
//Code words saved as globals for use in other modules

int CharPosLine=1; //Caracter position in line
int CharPosWord=0;

	Word_Count=1; //to get first word
	do	{		//Search for first character
		if (LineBuffer[1] ==13) {//13Carriage return so exit loop
			break;
		}
		if (Word_Count ==1)	{
			if (CharPosWord==0)	{ //its the letter code
				Inst_Code_Letter=LineBuffer[CharPosLine];
				CharPosLine++;
			}
			Inst_Code[CharPosWord]=LineBuffer[CharPosLine];
			CharPosWord++;
			if (LineBuffer[CharPosLine]==32) {
				CharPosLine++; //skip space
				CharPosWord=0; //reset
				Word_Count=2; //spot space at end of word
			}
		}
		if (Word_Count ==2)	{
			if (CharPosWord==0)	{ //its the letter code
				Dim_1_Letter=LineBuffer[CharPosLine];
				CharPosLine++;
			}
			Dim_1[CharPosWord]=LineBuffer[CharPosLine];
			CharPosWord++;
			if (LineBuffer[CharPosLine]==32) {
				CharPosLine++; //skip space
				CharPosWord=0; //reset
				Word_Count=3; //spot space at end of word
			}
		}
		if (Word_Count ==3)	{
			if (CharPosWord==0)	{ //its the letter code
				Dim_2_Letter=LineBuffer[CharPosLine];
				CharPosLine++;
			}
			Dim_2[CharPosWord]=LineBuffer[CharPosLine];
			CharPosWord++;
			if (LineBuffer[CharPosLine]==32) {
				CharPosLine++; //skip space
				CharPosWord=0; //reset
				Word_Count=4; //spot space at end of word
			}
		}
		if (Word_Count ==4)	{
			if (CharPosWord==0)	{ //its the letter code
				Dim_3_Letter=LineBuffer[CharPosLine];
				CharPosLine++;
			}
			Dim_3[CharPosWord]=LineBuffer[CharPosLine];
			CharPosWord++;
			if (LineBuffer[CharPosLine]==32) {
				CharPosLine++; //skip space
				CharPosWord=0; //reset
				Word_Count=5; //spot space at end of word
			}
		}
		if (Word_Count ==5)	{
			if (CharPosWord==0)	{ //its the letter code
				Dim_4_Letter=LineBuffer[CharPosLine];
				CharPosLine++;
			}
			Dim_4[CharPosWord]=LineBuffer[CharPosLine];
			CharPosWord++;
		}

		CharPosLine++;
	}while (LineBuffer[CharPosLine]!=13);//spot end of line

	//recast the data from ascii to integer or float
	iInst_Code = atoi(Inst_Code);
	fDim_1=atof(Dim_1);
	fDim_2=atof(Dim_2);
	fDim_3=atof(Dim_3);
	fDim_4=atof(Dim_4);

}//GetCodes

/********************************************************/
void FlushBuffer(void)	{
//Clears out all the data ready for a new Gcode line

int f;
	
	//LineBuffer
	for(f=49; f; --f) { //Flush line Buffer
		LineBuffer[f]=0;
	}
	if (f==0) LineBuffer[0]=0; //to clear remaining character
	
	//Inst_Code
	Inst_Code_Letter=0;
	for(f=7; f; --f) { //Flush line Buffer
		Inst_Code[f]=0;
	}
	if (f==0) Inst_Code[0]=0; //to clear remaining character
		
	//Dim_1	
	Dim_1_Letter=0;
	for(f=7; f; --f) { //Flush line Buffer
		Dim_1[f]=0;
	}
	if (f==0) Dim_1[0]=0; //to clear remaining character
	
	//Dim_2
	Dim_2_Letter=0;
	for(f=7; f; --f) { //Flush line Buffer
		Dim_2[f]=0;
	}
	if (f==0) Dim_2[0]=0; //to clear remaining character

	//Dim_3
	Dim_3_Letter=0;
	for(f=7; f; --f) { //Flush line Buffer
		Dim_3[f]=0;
	}
	if (f==0) Dim_3[0]=0; //to clear remaining character

	//Dim_4
	Dim_4_Letter=0;
	for(f=7; f; --f) { //Flush line Buffer
		Dim_4[f]=0;
	}
	if (f==0) Dim_4[0]=0; //to clear remaining character

}//FlushBuffer
/********************************************************/


