//***********************************************
//****************Stepper.c**********************
//***********************************************
//
//
//***********************************************
//****************Stepper.c**********************
//***********************************************

/*
Interrupt routines for the steppers have been included in this module
Running the Binary files seems to loose steps. Using inline code for the 
step time out works fine.
TODO HERE: Sort out the stepper interrupts!
*/

#include "Stepper.h"

//Module Globals

int Home=0;

int S_F_Code; //Stepper F code
int S_M_Code; //Stepper M code
int Feed_Rate= 15; //5 fast, 25 slow  Variable for inline step function
int XYFeed_Rate = 3000; //variable for interrupt step function.
int ZFeed_Rate  = 3000;

float Delta_X;	//Used to hold movement data
float Delta_Y;
float Delta_Z;

float Save_X=0;	//Last used coordinate saved here.
float Save_Y=0;
float Save_Z=0;
float Save_I=0;
float Save_J=0;

//Step value coordinates 
float X_Steps,Y_Steps,Z_Steps; 		//Float value of steps
float I_Steps, J_Steps;

long x_period;	//used for timer interrupt
long y_period;
long z_period;

signed long lX_Steps,lY_Steps,lZ_Steps;	//Long integer values of steps
signed long lI_Steps, lJ_Steps;
signed long lSaveSteps_X, lSaveSteps_Y, lSaveSteps_Z;
signed long lSaveSteps_I, lSaveSteps_J;

signed int BArc_x1, BArc_y1; //Bresenham Arc values
signed int BArc_x2, BArc_y2;

//Not used below
//char StepperOutputBuffer[16];
//char *StepperWriteReg = &StepperOutputBuffer[0];

/*
******************************************************************
		STEPPER INTERRUPTS
******************************************************************
The following interrupts toggle the state of the step pulse
Interrupt period high for 4 cycles then low until the next interrupt
one full cycle gives pulse frequency as interrupt period/2
*/ 

void _ISR _T3Interrupt(void)	{
//X-Axis step timer
	
	//Minimum here is two instructions, 1 is too short to be seen
	//by the stepper controller
	X_Step = 1;					//pulse high
	X_Step = 1;					//pulse high
	X_Step = 1;					//pulse high
	X_Step = 1;					//pulse high

	X_Step = 0;					//pluse low
	X_Step = 0;					//pluse low

	_T3IF = 0;	//reset the interrupt flag on exit
	_T3IE = 0;	//disable the T3 interrupt

}//T3Interrupt

//******************************************************************
void _ISR _T4Interrupt(void)	{
//Y-Axis step timer

	Y_Step = 1;					//pulse high
	Y_Step = 1;					//pulse high
	Y_Step = 1;					//pulse high
	Y_Step = 1;					//pulse high

	Y_Step = 0;					//pluse low
	Y_Step = 0;					//pluse low

	_T4IF = 0;	//reset the interrupt flag on exit
	_T4IE = 0;	//disable the T4 interrupt

}//T4Interrupt

//******************************************************************
void _ISR _T5Interrupt(void)	{
//Z-Axis step timer

	Z_Step = 1;					//pulse high
	Z_Step = 1;					//pulse high
	Z_Step = 1;					//pulse high
	Z_Step = 1;					//pulse high

	Z_Step = 0;					//pluse low
	Z_Step = 0;					//pluse low

	_T5IF = 0;	//reset the interrupt flag on exit
	_T5IE = 0;	//disable the T5 interrupt

}//T5Interrupt
/*
******************************************************************
*/
void Stepper_InitIO (void)	{

// Setup timers
/*
bit 5-4 TCKPS1:TCKPS0: Timer2 Input Clock Prescale Select bits
	11 = 1:256
	10 = 1:64
	01 = 1:8
	00 = 1:1
*/

//Timer 1
//32MHz clock T_delay=2/Fosc)*256*delay
// Bit    15 14 13 12 11 10 9 8     7 6 5 4  3 2 1 0      
//0b   80=1  0  0  0  0  0  0 0  30=0 0 1 1  0 0 0 0
T1CON = 0x8030;	//Set up timer 1 Fosc/2, prescaled 1:256 

//Timer 3		X-Axis timer
_T3IP = 1;	//Timer interrupt priority
T3CON = 0x8000;

//Timer 4		Y-Axis Timer
_T4IP = 2;	//Timer interrupt priority
T4CON = 0x8000;

//Timer 5		Z-Axis timer
_T5IP = 3;	//Timer interrupt priority
T5CON = 0x8000;

//Setup SPI 2 port - unused
SPI2STAT = 0;
//SPI2STAT = 0x7FFF;               // disable SPI port 2
//bx 7F=01111111 FF=11111111
//0 in bit 15 disables the module

//setup output pins PORTA
 TRISA=0; //configure all pins as outputs
//RA0		TMS  JTAG test mode select pin
//RA1		TCK	 JTAG test clock pin
//RA2		SCL2
//RA3		SDA2
//RA4		TDI  JTAG test data in
//RA5		TDO  JTAG test data out
//RA6		-
//RA7		-
//RA8
 
}//InitStepper

/*
****************************************************************
*/

void Stepper_LoadNewValues(int Feed_Code, int Machine_Code)	{
	
	S_F_Code = Feed_Code;
	S_M_Code = Machine_Code;

	//Set the feed Rate
	//Feed _Rate is timer ticks ie pulse frequency.
	// Feed_Rate value of 1  is fast 100"/min
	// Feed_Rate Value of 10 is slow 1"/min
	if(S_F_Code != 0)	{
		if (S_F_Code ==100)	{
			Feed_Rate = 1;
		}
		else	{
			Feed_Rate= 100-S_F_Code;
			Feed_Rate = (int) Feed_Rate/10;	//range of values is 1 to 10
		}
	}
	else (Feed_Rate = 5);

//*******************************************************
//TODO HERE:
	Feed_Rate = 5;//delete this line and debug feed rates
//*******************************************************

/*
	switch (Feed_Code)	{
		
		case 20 :
			Feed_Rate = Feed_20; //slow
			break;
		case 30 :
			Feed_Rate = Feed_30;
			break;
		case 40 :
			Feed_Rate = Feed_40;
			break;
		case 50 :
			Feed_Rate = Feed_50;
			break;
		case 60 :
			Feed_Rate = Feed_60;
			break;
	    case 70 :
			Feed_Rate = Feed_70;
			break;
		case 80 :
			Feed_Rate = Feed_80;
			break;
		case 90 :
			Feed_Rate = Feed_90;
			break;
		case 100 :
			Feed_Rate = Feed_100; //Fast
			break;
		default:
			Feed_Rate = Feed_10;  //if not specified then slowest speed
	}//switch
*/
}

/*
****************************************************************
*/
void Stepper_CalcStepsLine(int Units)	{

//Number of steps on each Axis
	switch (Units)	{
		case 20:	//Inches
			X_Steps =  GX * StepsPer_Inch;//numbers here are not rounded
			Y_Steps =  GY * StepsPer_Inch;//up or down!
			Z_Steps =  GZ * StepsPer_Inch;

			lX_Steps= f_to_sl(X_Steps);	//Float to long function at bottom of this 
			lY_Steps= f_to_sl(Y_Steps);	//section.	
			lZ_Steps= f_to_sl(Z_Steps);
			break;

		case 21:	//Metric
		default:	//Default metric
			X_Steps = GX * StepsPer_mm;
			Y_Steps = GY * StepsPer_mm;
			Z_Steps = GZ * StepsPer_mm;

			lX_Steps= f_to_sl(X_Steps);	//Float to long function at bottom of this 
			lY_Steps= f_to_sl(Y_Steps);	//section.	
			lZ_Steps= f_to_sl(Z_Steps);
			
	}//switch
}

/*
****************************************************************
*/
void Stepper_CalcStepsArc(int Units)	{

//step coordinate on each Axis
	switch (Units)	{
		case 20:	//Inches
			X_Steps =  GX * StepsPer_Inch;//numbers here are not rounded
			Y_Steps =  GY * StepsPer_Inch;//up or down!
			Z_Steps =  GZ * StepsPer_Inch;
			I_Steps =  GI_Val * StepsPer_Inch;
			J_Steps =  GJ_Val * StepsPer_Inch;

			lX_Steps= f_to_sl(X_Steps);	//Float to long function at bottom of this 
			lY_Steps= f_to_sl(Y_Steps);	//section.	
			lZ_Steps= f_to_sl(Z_Steps);
			lI_Steps= f_to_sl(I_Steps);
			lJ_Steps= f_to_sl(J_Steps);
			break;

		case 21:	//Metric
		default:	//Default metric
			X_Steps = GX * StepsPer_mm;
			Y_Steps = GY * StepsPer_mm;
			Z_Steps = GZ * StepsPer_mm;
			I_Steps = GI_Val * StepsPer_mm;
			J_Steps = GJ_Val * StepsPer_mm;

			lX_Steps= f_to_sl(X_Steps);	//Float to long function at bottom of this 
			lY_Steps= f_to_sl(Y_Steps);	//section.	
			lZ_Steps= f_to_sl(Z_Steps);
			lI_Steps= f_to_sl(I_Steps);
			lJ_Steps= f_to_sl(J_Steps);
			
	}//switch
}

/*
****************************************************************
*/
void Stepper_Run3DLine()	{
//all the variables in the following are available a global or local
	Bresenham_3D_Line(lX_Steps, lY_Steps, lZ_Steps,
					lSaveSteps_X, lSaveSteps_Y, lSaveSteps_Z);
}
/*
****************************************************************
*/
void Stepper_RunArcLine(int Direction)	{
//Direction = G2 circular interpolation = CLOCKWISE
//Direction = G3 circular interpolation = ANTI-CLOCKWISE

//int result;
int StartOctant = 0;
int EndOctant   = 0;
int ArcLen	= 0;
int Oct = 0;
//int i = 0;
double Radius = 0;
signed long CentreX = 0;
signed long CentreY = 0;

signed long End_X = 0;
signed long End_Y = 0;
signed long Start_X = 0;
signed long Start_Y = 0;

//Calculate radius and centre coordinates here
//The tool is already at the start position for the arc

//////////////////////////////////////////////
//set up some dummy coordinates for test
//////////////////////////////////////////////
//lI_Steps = 10;
//lJ_Steps = 0;
//lSaveSteps_X = 1;
//lSaveSteps_Y = 15;
//lX_Steps = 11;
//lY_Steps = 25;
//Direction = 2;
//////////////////////////////////////////////

//Start coordinates are the saved ones
//lSaveSteps_X, lSaveSteps_Y, I and J give the x and y offset for the circle
//Radius of the arc is root(I^2 + J^2)
	
	if ((lI_Steps == 0)|(lJ_Steps == 0))	{
		//no need for calc
		if (lI_Steps != 0) Radius = abs(lI_Steps);
		if (lJ_Steps != 0) Radius = abs(lJ_Steps);
	} 
	else	{
		Radius =lI_Steps * lI_Steps +lJ_Steps * lJ_Steps;
		Radius = sqrt(Radius);
	}
	
	CentreX = lSaveSteps_X + lI_Steps;
	CentreY = lSaveSteps_Y + lJ_Steps;

	//if (Direction==3)	{
	Start_X = Start_X - lI_Steps;//to offset circle to origin.
	Start_Y = Start_Y - lJ_Steps;
	End_X = lX_Steps - CentreX; //to offset the circle to the origin
	End_Y = lY_Steps - CentreY;
	//}
	StartOctant = Get_Start_Octant(Start_X, Start_Y, Direction);
	EndOctant = Get_End_Octant(End_X, End_Y, Direction);

	//TODO HERE: 
	//May need to do a check on the end point of the arc to ensure
	//it is a coordinate on the circle
	//Do check here 
	//
	
	if ((lSaveSteps_X == lX_Steps)&(lSaveSteps_Y == lY_Steps)) { //its a full circle
		if (Direction == 2)	{
			//circle in G2 Clockwise dir
			Move_To_Start(CentreX, CentreY);
			B_Arc_CW_Oct2468(CentreX, CentreY, Radius, 8);
			B_Arc_CW_Oct1357(CentreX, CentreY, Radius, 7);
			B_Arc_CW_Oct2468(CentreX, CentreY, Radius, 6);
			B_Arc_CW_Oct1357(CentreX, CentreY, Radius, 5);
			B_Arc_CW_Oct2468(CentreX, CentreY, Radius, 4);
			B_Arc_CW_Oct1357(CentreX, CentreY, Radius, 3);
			B_Arc_CW_Oct2468(CentreX, CentreY, Radius, 2);
			B_Arc_CW_Oct1357(CentreX, CentreY, Radius, 1);
		}
		if (Direction == 3)	{
			//circle in G3 Anti-clockwise dir
			Move_To_Start(CentreX, CentreY);
			B_Arc_CCW_Oct1357(CentreX, CentreY, Radius, 1);
			B_Arc_CCW_Oct2468(CentreX, CentreY, Radius, 2);
			B_Arc_CCW_Oct1357(CentreX, CentreY, Radius, 3);
			B_Arc_CCW_Oct2468(CentreX, CentreY, Radius, 4);
			B_Arc_CCW_Oct1357(CentreX, CentreY, Radius, 5);
			B_Arc_CCW_Oct2468(CentreX, CentreY, Radius, 6);
			B_Arc_CCW_Oct1357(CentreX, CentreY, Radius, 7);
			B_Arc_CCW_Oct2468(CentreX, CentreY, Radius, 8);
		}
		StartOctant = EndOctant; //to ensure it does not drop into the loop below
	}

	Oct = StartOctant;				//initialise
	while (Oct != EndOctant)	{
		if (Direction == 3)	{
			Oct = StartOctant + ArcLen;
			if (Oct > 8) {
				Oct = 1;
				ArcLen = 1;
				StartOctant = 0;		//reset Oct if it runs over 
			}

			//cut octant 
			RunOctant( CentreX, CentreY , Radius, Oct, 3);
		
			ArcLen += 1;	//Adding to run anti-clock
		}//if
		

		if (Direction == 2)	{
			Oct = StartOctant + ArcLen;
			if (Oct < 1) {
				Oct = 8;
				ArcLen = 0;
				StartOctant = 8;		//reset Oct if it runs over 
			}

			//cut octant
 			RunOctant( CentreX, CentreY , Radius, Oct, 2);
		
			ArcLen -= 1;	//Adding to run anti-clock
		}//if
	}//while	
}
/*
****************************************************************
*/
void Move_To_Start(signed long CentreX,signed long CentreY) {

//this function sets the tool at the start of the circle - 3-oClock

signed long CircleStartX = 0;
signed long CircleStartY = 0;

//if lI_Steps is +ve then we are at 9 o-clock - lJ_Steps is zero
	if ((lI_Steps >0)&(lJ_Steps==0))	{
		CircleStartX = CentreX + lI_Steps;
		CircleStartY = CentreY;
		Bresenham_3D_Line(CircleStartX, CircleStartY, lZ_Steps,
				lSaveSteps_X, lSaveSteps_Y, lSaveSteps_Z);
		//update saved values
		lX_Steps = CircleStartX;
		//Save_X =
		lY_Steps = CircleStartY;
		//Save_Y =
	}
//if lI_Steps is -ve then we are already set at the start point - lJ_Steps is zero
	if ((lI_Steps <0)&(lJ_Steps==0))	{
		//no position change
	}

//if lJ_Steps is +ve then we are at 6 o-clock - lI_Steps is zero
	if ((lI_Steps ==0)&(lJ_Steps >0))	{
		CircleStartX = CentreX + lJ_Steps;
		CircleStartY = CentreY;
		Bresenham_3D_Line(CircleStartX, CircleStartY, lZ_Steps,
				lSaveSteps_X, lSaveSteps_Y, lSaveSteps_Z);
		//update saved values
		lX_Steps = CircleStartX;
		//Save_X =
		lY_Steps = CircleStartY;
		//Save_Y =
	}
//if lJ_Steps is -ve then we are at 12 o-clock - lI_Steps is zero
	if ((lI_Steps ==0)&(lJ_Steps <0))	{
		CircleStartX = CentreX + lJ_Steps;
		CircleStartY = CentreY;
		Bresenham_3D_Line(CircleStartX, CircleStartY, lZ_Steps,
				lSaveSteps_X, lSaveSteps_Y, lSaveSteps_Z);
		//update saved values
		lX_Steps = CircleStartX;
		//Save_X =
		lY_Steps = CircleStartY;
		//Save_Y =
	}
}
/*
****************************************************************
*/
void RunOctant(signed int xmid, signed int ymid,
						 signed int r, int Octant, int Direction)	{


	if (Direction == 3)	{	//Anti-Clockwise
	
		 	switch (Octant)	{
				case 1:
					B_Arc_CCW_Oct1357(xmid, ymid, r, 1);
					break;
				case 2:
					B_Arc_CCW_Oct2468(xmid, ymid, r, 2);
					break;
				case 3:
					B_Arc_CCW_Oct1357(xmid, ymid, r, 3);
					break;
				case 4:
					B_Arc_CCW_Oct2468(xmid, ymid, r, 4);
					break;
				case 5:
					B_Arc_CCW_Oct1357(xmid, ymid, r, 5);
					break;
				case 6:
					B_Arc_CCW_Oct2468(xmid, ymid, r, 6);
					break;
				case 7:
					B_Arc_CCW_Oct1357(xmid, ymid, r, 7);
					break;
				case 8:
					B_Arc_CCW_Oct2468(xmid, ymid, r, 8);
					break;
			}//switch
	}//if

	if (Direction == 2)	{	//Clockwise

		switch (Octant)	{
				case 1:
					B_Arc_CW_Oct1357(xmid, ymid, r, 1);
					break;
				case 2:
					B_Arc_CW_Oct2468(xmid, ymid, r, 2);
					break;
				case 3:
					B_Arc_CW_Oct1357(xmid, ymid, r, 3);
					break;
				case 4:
					B_Arc_CW_Oct2468(xmid, ymid, r, 4);
					break;
				case 5:
					B_Arc_CW_Oct1357(xmid, ymid, r, 5);
					break;
				case 6:
					B_Arc_CW_Oct2468(xmid, ymid, r, 6);
					break;
				case 7:
					B_Arc_CW_Oct1357(xmid, ymid, r, 7);
					break;
				case 8:
					B_Arc_CW_Oct2468(xmid, ymid, r, 8);
					break;
			}//switch
	}//if

}

/*
****************************************************************
*/
void Stepper_SaveValues()	{

//Tidy up after Gcode instruction.
//Save current dimension and step coordinates 

	lSaveSteps_X = lX_Steps;
	lSaveSteps_Y = lY_Steps;
	lSaveSteps_Z = lZ_Steps;

	Save_X = GX;
	Save_Y = GY;
	Save_Z = GZ;

//Reset globals
	Delta_X=0;
	Delta_Y=0;
	Delta_Z=0;
	X_Steps=0;
	Y_Steps=0;
	Z_Steps=0; 
	I_Steps=0;
	J_Steps=0;
	lX_Steps=0;
	lY_Steps=0;
	lZ_Steps=0;	
	lI_Steps=0;
	lJ_Steps=0;
	BArc_x1=0;
 	BArc_y1=0;
 	BArc_x2=0;
	BArc_y2=0;
}

/*
****************************************************************
*/
void Bresenham_2D_Line(int x0, int y0, int x1, int y1)	{

//UNTESTED and code is not called by any function
//This may be worth implementing if there are two active axis
//in the GCode

int steep, t;
int deltax, deltay, error;
int x, y;
int ystep;

	if (abs(y1-y0) > abs(x1-x0)) {
		steep = 1;
	}
	else steep = 0;


	if (steep)	{	// swap x and y
		t=x0; x0=y0; y0=t;
		t=x1; x1=y1, y1=t;
	}

	if (x0 > x1)	{	//swap ends
		t=x0; x0=x1; x1=t;
		t=y0; y0=y1; y1=t;
	}
	
	deltax = x1-x0;
	deltay = abs(y1-y0);
	error = 0;
	y = y0;

	if  (y0 < y1) ystep = 1; else ystep = -1;
	for (x=x0; x<x1; x++)	{

	//	if (steep) plot(y,x); else plot(x,y);
		//if x and y have been swapped plot (y,x)
		//if x and y not  been swapped plot (x,y)
		if (steep) SingleStep(0,1,0); else SingleStep(1,0,0);		

		error += deltay;
		if ((error<<1) >= deltax)	{
			y += ystep;
			error -= deltax;
		}//if
	}//for

}//line

/*
****************************************************************
*/
void Bresenham_3D_Line(signed int x1, signed int y1, signed int z1, 
						signed int x2, signed int y2, signed int z2)	{

int MajorAxis = 0;
signed int incX = 1;
signed int incY = 1;
signed int incZ = 1;
signed long DeltaX = x2-x1;
signed long DeltaY = y2-y1;
signed long DeltaZ = z2-z1;
signed long TwoDeltaX = DeltaX + DeltaX;
signed long TwoDeltaY = DeltaY + DeltaY;
signed long TwoDeltaZ = DeltaZ + DeltaZ;
int currentX = x1;
int currentY = y1;
int currentZ = z1;
int ErrorX = 0;
int ErrorY = 0;
int ErrorZ = 0;
int moveX = 0;
int moveY = 0;
int moveZ = 0;

	if (!GX_On) {	//checks to see if the code mentions X
		DeltaX = 0;	//if not mentioned then no change
		TwoDeltaX=0;
	}
	if (!GY_On)	{
		DeltaY = 0;
		TwoDeltaY=0;
	}
	if (!GZ_On)	{
		DeltaZ = 0;
		TwoDeltaZ=0;
	}

	//if there is no delta, current step coord is same as saved value
	if (DeltaX == 0) {
		x1=x2;
		lX_Steps=lSaveSteps_X;
		//SX = Save_X;
		GX = Save_X;
	}
	if (DeltaY == 0) {
		y1=y2;
		lY_Steps=lSaveSteps_Y;
		//SY = Save_Y;
		GY = Save_Y;
	}
	if (DeltaZ == 0) {
		z1=z2;
		lZ_Steps=lSaveSteps_Z;
	//	SZ = Save_Z;
		GZ = Save_Z;
	}

	if (DeltaX < 0) {
		X_Dir = 1;		//Set X Direction based on sign of Delta value
		incX = -1;
		DeltaX = -DeltaX;
		TwoDeltaX = -TwoDeltaX;
	}
	else 
		X_Dir = 0;

	if (DeltaY < 0) {
		Y_Dir = 1;		//Set Y Direction based on sign of Delta value
		incY = -1;
		DeltaY = -DeltaY;
		TwoDeltaY = -TwoDeltaY;
	}
	else
		Y_Dir = 0;

	if (DeltaZ < 0) {
		Z_Dir = 0;		//Set Z Direction based on sign of Delta value
		incZ = -1;
		DeltaZ = -DeltaZ;
		TwoDeltaZ = -TwoDeltaZ;
	}
	else
		Z_Dir = 1;


	if ((DeltaZ >= DeltaX)&(DeltaZ >= DeltaY))(MajorAxis = 1);
	if ((DeltaX >= DeltaZ)&(DeltaX >= DeltaY))(MajorAxis = 2);	
	if ((DeltaY >= DeltaX)&(DeltaY >= DeltaZ))(MajorAxis = 3);
	
	if ((DeltaX ==0)&(DeltaY==0)&(DeltaZ==0)) MajorAxis = 0; 	

	if (MajorAxis == 0) {
		//error trap if required
		//if MajorAxis is 0 then it drops out of this code with no action
	}

//case 1
	if (MajorAxis == 1)	{
		do	{
			currentZ += incZ;
			ErrorX += TwoDeltaX;
			ErrorY += TwoDeltaY;

			if (ErrorX > DeltaZ)	{
				currentX += incX;
				ErrorX -= TwoDeltaZ;
				moveX = 1;
			}
			if (ErrorY > DeltaZ)	{
				currentY += incY;
				ErrorY -= TwoDeltaZ;
				moveY = 1;
			}
/*
//Wait while step interrupts are enabled ie busy.
//Uncomment the following lines to use the interrupt step code
//do the same for case 2 and 3.
			while (_T3IE == 1);
			while (_T4IE == 1);
			while (_T5IE == 1);
			SingleStepInt(moveX,moveY,1);
*/
			SingleStep(moveX,moveY,1); //comment out this line to use interrupt code
			moveX = 0, moveY = 0, moveZ = 0;

		} while(currentZ != z2);
		
	}//if

//case 2
	if (MajorAxis == 2)	{
		do	{
			currentX += incX;
			ErrorZ += TwoDeltaZ;
			ErrorY += TwoDeltaY;

			if (ErrorZ > DeltaX)	{
				currentZ += incZ;
				ErrorZ -= TwoDeltaX;
				moveZ = 1;
			}
			if (ErrorY > DeltaX)	{
				currentY += incY;
				ErrorY -= TwoDeltaX;
				moveY = 1;
			}
			//see note above
			//hold while interrupt is busy
			//while (_T3IE == 1);
			//while (_T4IE == 1);
			//while (_T5IE == 1);
			//SingleStepInt(1,moveY,moveZ);
			SingleStep(1,moveY,moveZ);
			moveX = 0, moveY = 0, moveZ = 0;
		

		} while(currentX != x2);
	}//if

//case 3
	if (MajorAxis == 3)	{
		if (currentY != y2)	{ //here current y already equals y2 so skip
			do	{
				currentY += incY;
				ErrorX += TwoDeltaX;
				ErrorZ += TwoDeltaZ;

				if (ErrorX > DeltaY)	{
					currentX += incX;
					ErrorX -= TwoDeltaY;
					moveX = 1;	
				}
				if (ErrorZ > DeltaY)	{
					currentZ += incZ;
					ErrorZ -= TwoDeltaY;
					moveZ = 1;
				}
				//see note above
				//hold while interrupt is busy
				//while (_T3IE == 1);
				//while (_T4IE == 1);
				//while (_T5IE == 1);
				//SingleStepInt(moveX,1,moveZ);
				SingleStep(moveX,1,moveZ);
				moveX = 0, moveY = 0; moveZ = 0;
			
			} while(currentY != y2);
		}//if
	}//if

	
}

/*
****************************************************************
*/
void SingleStepInt(int xs, int ys, int zs)	{
//EDIT THIS ONE FOR THE STEP INTERRUPTS
int i;

	if ( xs)	{
		SingleStepX(XYFeed_Rate, X_Dir);
	}
	if ( ys)	{
		SingleStepY(XYFeed_Rate, Y_Dir);
	}
	if ( zs)	{
		for (i=0; i<=15; ++i){
			while (_T5IE == 1);
			SingleStepZ(ZFeed_Rate, Z_Dir);
		}
	}

}

/*
****************************************************************
*/
void SingleStep(int xs, int ys, int zs)	{
//inline step code, not used if you run with interrupt timers
//for stepping
int i;

		if ( xs) X_Step = 1;					//pulse high
		if ( ys) Y_Step = 1;
		if ( zs) Z_Step = 1;

		TMR1 = 0; while(TMR1<Feed_Rate);

		X_Step = 0;						//pluse low
		Y_Step = 0;	
		Z_Step = 0;
		
		//TODO HERE:
		//setting XY&Z_Step to zero (above) should take the pulse low
		PORTAbits.RA0 = 0;	
		PORTAbits.RA2 = 0;
		PORTAbits.RA4 = 0;
		/*On the Explorer 16 dev. board
		for some reason the port a bits do not switch reliably if they
		they are only switched once.
		Debug portA.
		*/	
		TMR1 = 0; while(TMR1<Feed_Rate);

	
	//CHECK on E-STOP here!

}//SingleStep

/*
****************************************************************
*/
long f_to_ul(float fNumber)	{
//Function returns a rounded unsigned long value for fNumber
float IntNumber, Fraction;
long TempNum;

	Fraction=modf(fNumber,&IntNumber);//split float into Int and fraction
	TempNum = (long)IntNumber;
	
	if (Fraction >= 0.5)	{
		TempNum += 1;
		return TempNum;
	}
	else
		return TempNum;

}
/*
****************************************************************
*/
signed long f_to_sl(float fNumber)	{
//Function returns a rounded signed long value for fNumber
float IntNumber, Fraction;
signed long TempNum;

	Fraction=modf(fNumber,&IntNumber);//split float into Int and fraction
	TempNum = (signed long)IntNumber;
	
	if (Fraction >= 0.5)	{
		TempNum += 1;	
	}
	if (Fraction <= -0.5)	{
		TempNum -= 1;
	}

	return TempNum;
}

/*
****************************************************************************
*/

/*
Image:Bresenham circle3.png
Rasterisation of a circle by the Bresenham algorithm

The approach for the Circle Variant shown here is also not originally from Bresenham,
see again references to Pitteway and van Aken below. The algorithm starts accordingly
with the circle equation x+y=r. Again we start by considering only the first octant.
Here you want to draw a curve which starts at point (r,0) and then proceeds to the
top left, up to reaching the angle of 45.

The fast direction here is the y direction. You always do a step in the positive y
direction (upwards), and every now and then you also have to do a step in the
slow direction, the negative x direction.

The frequent computations of squares in the circle equation, trigonometric expressions
or square roots can again be avoided by dissolving everything into single steps and
recursive computation of the quadratic terms from the preceding ones.

From the circle equation you get to the transformed equation 0=x+y-r with r to be
computed only a single time during initialisation,
x=(xpreceding-1)=xpreceding-2*xpreceding+1 (according for y),
where x (or xpreceding) is kept as an own variable.
Additionally you need to add the mid point coordinates when setting a pixel.
These frequent integer additions do not limit the performance much, as you spare those
square (root) computations in the inner loop in turn. Again the zero in the transformed
circle equation is replaced by the error term.

The initialization of the error term is derived from an offset of  pixel at the start.
Until the intersection with the perpendicular line, this leads to an accumulated value
of r in the error term, so that this value is used for initialisation.

The following implementation is shown here only for the first octant, and again the 
other octants need sign changes for x and/or y and the swapping of x and y.
An easy expansion for full circles, as it is possible for graphics displays,
but not for plotters, is added in the comments.
*/

/*
******************************************************************************
*/

void B_Arc_CCW_Oct1357(signed int x_mid, signed int y_mid, signed int r, int Octant)	{
//The tool must be at the start coordinates before we call this function.

//G3 circular interpolation - Anti-clockwise

signed int error = r;
signed int x = r;
signed int y = 0;
signed int dx,dy = 0;
int moveX = 0;
int moveY = 0;

//step loop: always a step in longest direction,
//only step in other direction when error is large enough.

while (y < x)	{
   // y is longest (positive y direction)
   dy = y*2+1;
   y = y+1;

   switch (Octant)	{
		case 1:
   			Y_Dir=1; // +ve direction
 			moveY = 1; 
			break;
		case 3:	
			X_Dir=0; // -ve direction x and y swapped
			moveX = 1;
			break;
		case 5:
			Y_Dir=0; // -ve direction
			moveY = 1;
			break;
		case 7:
			X_Dir=1; // +ve direction x and y swapped
			moveX = 1;
			break;
	}//switch
   error = error-dy;

   if (error<0)	{
      //step in the shortest direction (negative x direction)
      dx = 1-x*2; 
      x = x-1;
	  switch (Octant)	{
			case 1:
   				X_Dir=0; // -ve direction
				moveX = 1;
				break;
			case 3:	
				Y_Dir=0; // -ve direction x and y swapped
				moveY = 1;
				break;
			case 5:
				X_Dir=1; // +ve direction
				moveX = 1;
				break;
			case 7:
				Y_Dir=1; // +ve direction x and y swapped
				moveY = 1;
				break;
	  }//switch
      error = error-dx;
   }//if

	SingleStep(moveX,moveY,0);
    moveX=0,moveY=0;

 }//while

}//B_Arc_CCW_Oct1357

/*
********************************************************************************
*/
//DEBUG this - works as CCW as it is!
void B_Arc_CW_Oct2468(signed int x_mid, signed int y_mid, signed int r, int Octant)	{
//The tool must be at the start coordinates before we call this function.

//G2 circular interpolation - Clockwise

signed int error = r;

signed int x = r;
signed int y = 0;
signed int dx,dy = 0;
int moveX = 0;
int moveY = 0;

//step loop: always a step in longest direction,
//only step in other direction when error is large enough.

while (y < x)	{
   // y is longest (positive y direction)
   dy = y*2+1;
   y = y+1;

   switch (Octant)	{
		case 2:
   			X_Dir=1; // +ve direction y and x swapped
			moveX = 1;
			break;
		case 4:	
			Y_Dir=1; // +ve direction
			moveY = 1;
			break;
		case 6:
			X_Dir=0; // -ve direction y and x swapped
			moveX = 1;
			break;
		case 8:
			Y_Dir=0; // -ve direction
			moveY = 1;
			break;
	}//switch
   error = error-dy;

   if (error<0)	{
      //step in the shortest direction (negative x direction)
      dx = 1-x*2; 
      x = x-1;
	  switch (Octant)	{
			case 2:
   				Y_Dir=0; // -ve direction y and x swapped
				moveY = 1;
				break;
			case 4:	
				X_Dir=1; // +ve direction
				moveX = 1;
				break;
			case 6:
				Y_Dir=1; // +ve direction y and x swapped
				moveY = 1;
				break;
			case 8:
				X_Dir=0; // -ve direction
				moveX = 1;
				break;
	  }//switch
      error = error-dx;
   }//if

	SingleStep(moveX,moveY,0);
    moveX=0,moveY=0;

 }//while

}//B_Arc_CW_Oct2468

/*
********************************************************************************
*/

void B_Arc_CW_Oct1357(signed int x_mid, signed int y_mid, signed int r, int Octant)	{
//The tool must be at the start coordinates before we call this function.

//G2 circular interpolation - Clockwise

signed int error = r;
//signed int r2 = r*r; //r squared
// root 2 = 1.414213562
float rxy = r/1.4142;
signed int x = 0;
signed int y = 0;
signed int dx,dy = 0;
int moveX = 0;
int moveY = 0;

//start condition: x = y = r/root2
x = (signed int)rxy;
y = x;

//step loop: always a step in longest direction,
//only step in other direction when error is large enough.

while (y > 0)	{
   // y is longest (positive y direction)
   dy = y*2+1;
   y = y-1;

   switch (Octant)	{
		case 1:
 			Y_Dir=0; //set direction
			moveY = 1;
		  	break;
		case 3:
 			X_Dir=1; //x and y swapped
			moveX = 1;
		  	break;
		case 5:
 			Y_Dir=1; //set direction
			moveY = 1;
		  	break;
		case 7:
 			X_Dir=0; //x and y swapped
			moveX = 1;
		  	break;
   }//switch

   error = error-dy;

   if (error<0)	{
      //step in slow direction (negative x direction)
	  dx = 1-x*2; 	//in Assembler implementation *2 per Shift
	  if (x < r)	{
      	x = x+1;
	  
		switch (Octant)	{
			case 1:
 				X_Dir=1; //set direction
				moveX = 1;
		  		break;
			case 3:
 				Y_Dir=1; //x and y swapped
				moveY = 1;
		  		break;
			case 5:
 				X_Dir=0; //set direction
				moveX = 1;
		  		break;
			case 7:
 				Y_Dir=0; //x and y swapped
				moveY = 1;
		  		break;
		}//switch
	  }//if
      error = error-dx;
   }//if


  
	SingleStep(moveX,moveY,0);
    moveX=0,moveY=0;

 }//while

}//B_Arc_CW_Oct1357

/*
********************************************************************************
*/

void B_Arc_CCW_Oct2468(signed int x_mid, signed int y_mid, signed int r, int Octant)	{
//The tool must be at the start coordinates before we call this function.

//G3 circular interpolation - Anti-clockwise

signed int error = r;
//signed int r2 = r*r; //r squared
// root 2 = 1.414213562
float rxy = r/1.4142;
signed int x = 0;
signed int y = 0;
signed int dx,dy = 0;
int moveX = 0;
int moveY = 0;

//start condition: x = y = r/root2
x = (signed int)rxy;
y = x;

//step loop: always a step in longest direction,
//only step in other direction when error is large enough.

while (y > 0)	{
   // y is longest (positive y direction)
   dy = y*2+1;
   y = y-1;

   switch (Octant)	{
		case 2:
 			X_Dir=0; //x and y swapped
			moveX = 1;
		  	break;
		case 4:
 			Y_Dir=0; 
			moveY = 1;
		  	break;
		case 6:
 			X_Dir=1; //x and y swapped
			moveX = 1;
		  	break;
		case 8:
 			Y_Dir=1; 
			moveY = 1;
		  	break;
   }//switch
   error = error-dy;

   if (error<0)	{
      //step in slow direction (negative x direction)
	  dx = 1-x*2; 	//in Assembler implementation *2 per Shift
	  if (x < r)	{
      	x = x+1;
	  
		switch (Octant)	{
			case 2:
 				Y_Dir=1; //x and y swapped
				moveY = 1;
		  		break;
			case 4:
 				X_Dir=0;
				moveX = 1;
		  		break;
			case 6:
 				Y_Dir=0; //x and y swapped
				moveY = 1;
		  		break;
			case 8:
 				X_Dir=1;
				moveX = 1;
		  		break;
		}//switch
	  }//if

      error = error-dx;
   }//if

	SingleStep(moveX,moveY,0); 
    moveX=0,moveY=0;

 }//while
}//B_Arc_CCW_Oct2468

/*
********************************************************************************
*/

int Get_Start_Octant(signed int x, signed int y, int G2orG3)	{
/*           |					
    \3 (-x+y)| 2 (+x+y) /	 Octant 1  +x+y		x > y
	   \	 |      /				4  -x+y		x > y
  4 (-x+y)\  |  /  1 (+x+y)			5  -x-y		x > y	
  -----------|------------			8  +x-y		x > y
  5 (-x-y) / |  \  8 (+x-y)			
	    /    |      \				2  +x+y		x < y
	/6 (-x-y)| 7 (+x-y) \			3  -x+y		x < y
			 |						6  -x-y		x < y
									7  +x-y		x < y
*/


//G2 circular interpolation - Clockwise
//G3 circular interpolation - Anti-clockwise

int result = 0;

	if (abs(x) > abs(y)) { //Octant is 1,4,5,8
		if (x > 0)	{ // is 1,8
			if ((y > 0)&(G2orG3==3))	{
				 result = 1;
			}
			else result = 8;	
		}

		if (x < 0)	{ // is 4,5	
			if (y > 0)	{
				 result = 4;
			}
			else result = 5;	
		}
	}//if

	if (abs(x) < abs(y)) { //Octant is 2,3,6,7
		if (x > 0)	{ // is 2,7
			if (y > 0)	{
				 result = 2;
			}
			else result = 7;	
		}

		if (x < 0)	{ // is 3,6	
			if (y > 0)	{
				 result = 3;
			}
			else result = 6;	
		}
	}//if

	if ((x == 0)|(y == 0))	{ //if1
		if (x == 0)	{ // if2
			if (y > 0)	{	//if4
				if (G2orG3==2){
				 result = 2;
				}
				if (G2orG3==3){
				 result = 3;
				}
			}//if4
			if (y < 0)	{	//if5
				if (G2orG3==2){
				 result = 6;
				}
				if (G2orG3==3){
				 result = 7;
				}
			}//if5
		}//if2

		if (y == 0)	{ // if3
			if (x > 0)	{	//if6
				if (G2orG3==2){
				 result = 8;
				}
				if (G2orG3==3){
				 result = 1;
				}
			}//if6
			if (x < 0)	{	//if7
				if (G2orG3==2){
				 result = 4;
				}
				if (G2orG3==3){
				 result = 5;
				}
			}//if7
		}//if3

	}//if1

	if (abs(x) == abs(y))	{ //if1
		if (x > 0)	{ // is 1,8  if2
			if (y >= 0)	{	//if4
				if (G2orG3==2){
				 result = 1;
				}
				if (G2orG3==3){
				 result = 2;
				}
			}//if4
			if (y < 0)	{	//if5
				if (G2orG3==2){
				 result = 7;
				}
				if (G2orG3==3){
				 result = 8;
				}
			}//if5
		}//if2

		if (x < 0)	{ // is 4,5	 if3
			if (y > 0)	{	//if6
				if (G2orG3==2){
				 result = 3;
				}
				if (G2orG3==3){
				 result = 4;
				}
			}//if6
			if (y < 0)	{	//if7
				if (G2orG3==2){
				 result = 5;
				}
				if (G2orG3==3){
				 result = 6;
				}
			}//if7
		}//if3

	}//if1

	return result;

}//Get_Start_Octant

/*
********************************************************************************
*/

int Get_End_Octant(signed int x, signed int y, int G2orG3)	{
/*           |					
    \3 (-x+y)| 2 (+x+y) /	 Octant 1  +x+y		x > y
	   \	 |      /				4  -x+y		x > y
  4 (-x+y)\  |  /  1 (+x+y)			5  -x-y		x > y	
  -----------|------------			8  +x-y		x > y
  5 (-x-y) / |  \  8 (+x-y)			
	    /    |      \				2  +x+y		x < y
	/6 (-x-y)| 7 (+x-y) \			3  -x+y		x < y
			 |						6  -x-y		x < y
									7  +x-y		x < y
*/


//G2 circular interpolation - Clockwise
//G3 circular interpolation - Anti-clockwise

int result = 0;

	if (abs(x) > abs(y)) { //Octant is 1,4,5,8
		if (x > 0)	{ // is 1,8
			if ((y > 0)&(G2orG3==3))	{
				 result = 1;
			}
			else result = 8;	
		}

		if (x < 0)	{ // is 4,5	
			if (y > 0)	{
				 result = 4;
			}
			else result = 5;	
		}
	}//if

	if (abs(x) < abs(y)) { //Octant is 2,3,6,7
		if (x > 0)	{ // is 2,7
			if (y > 0)	{
				 result = 2;
			}
			else result = 7;	
		}

		if (x < 0)	{ // is 3,6	
			if (y > 0)	{
				 result = 3;
			}
			else result = 6;	
		}
	}//if

	if ((x == 0)|(y == 0))	{ //if1
		if (x == 0)	{ // if2
			if (y > 0)	{	//if4
				if (G2orG3==2){
				 result = 3;
				}
				if (G2orG3==3){
				 result = 2;
				}
			}//if4
			if (y < 0)	{	//if5
				if (G2orG3==2){
				 result = 7;
				}
				if (G2orG3==3){
				 result = 6;
				}
			}//if5
		}//if2

		if (y == 0)	{ // if3
			if (x > 0)	{	//if6
				if (G2orG3==2){
				 result = 1;
				}
				if (G2orG3==3){
				 result = 8;
				}
			}//if6
			if (x < 0)	{	//if7
				if (G2orG3==2){
				 result = 5;
				}
				if (G2orG3==3){
				 result = 4;
				}
			}//if7
		}//if3

	}//if1

	if (abs(x) == abs(y))	{ //if1
		if (x > 0)	{ // is 1,8  if2
			if (y >= 0)	{	//if4
				if (G2orG3==2){
				 result = 1;
				}
				if (G2orG3==3){
				 result = 2;
				}
			}//if4
			if (y < 0)	{	//if5
				if (G2orG3==2){
				 result = 7;
				}
				if (G2orG3==3){
				 result = 8;
				}
			}//if5
		}//if2

		if (x < 0)	{ // is 4,5	 if3
			if (y > 0)	{	//if6
				if (G2orG3==2){
				 result = 3;
				}
				if (G2orG3==3){
				 result = 4;
				}
			}//if6
			if (y < 0)	{	//if7
				if (G2orG3==2){
				 result = 5;
				}
				if (G2orG3==3){
				 result = 6;
				}
			}//if7
		}//if3

	}//if1

	return result;

}//Get_End_Octant


/*
***************************************************************************
*/

/*
TODO HERE: sort out arc of any size

The implementations above always only draw complete octants or circles.
If you want to draw only a certain arc from an angle a to an angle , you have
to implement it in a way to first calculate the x and y coordinates of these
end points, where you inevitably have to resort to trigonometric or square root
computations (see Methods of computing square roots). Then you run the
Bresenham algorithm over the complete octant or circle and set the pixels only
if they fall into the wanted interval. After finishing this arc, you can abort
the algorithm prematurely.

By scaling the drawn x and y values (and horizontal or vertical line expansion,
respectively) you can produce even ellipses parallel to the x or y axis. For this,
you use the circle algorithm with the smaller ellipse axis as radius and add a
value in the other direction, which again is computed through another Bresenham
line algorithm increasing from the pole to the equator. As the ellipse has to be
elongated into the longer axis direction, you dont set single pixels anymore,
but have to draw lines (though simple ones, only horizontal or vertical) from
the previous to the next point.

A general ellipse can be derived from such an axis-parallel one by application
of a shearing operation on it. Again you use an additional Bresenham line algorithm
to compute the offset increasing in one of the axis directions and to let it
contribute to every drawn coordinate.
*/

/*
****************************************************************************
	The next three functions are used to start the step interrupt
*/

void SingleStepX(long x_period,int dir)	{
//sets up the direction and interrupt time period

	if (dir ==1) {X_Dir=1;} //set direction on X axis
	else {
		X_Dir = 0;
		X_Dir = 0;
	}

	PR3 = x_period-1;	//set the period register

	_T3IF = 0;	//clear the interrupt flag
	TMR3  = 0;	//Clear the timer to zero
	_T3IE = 1;	//enable the T3 interrupt

}

void SingleStepY(long y_period, int dir)	{
 //sets up the direction and interrupt time period

	if (dir ==1) {Y_Dir=1;} //set direction on X axis
	else {
		Y_Dir = 0;
		Y_Dir = 0;
	}

	PR4 = y_period-1;	//set the period register

	_T4IF = 0;	//clear the interrupt flag
	TMR4  = 0;	//Clear the timer to zero
	_T4IE = 1;	//enable the T4 interrupt

}

void SingleStepZ(long z_period, int dir)	{
//sets up the direction and interrupt time period

	if (dir ==1) {Z_Dir=1;} //set direction on X axis
	else {
		Z_Dir = 0;
		Z_Dir = 0;
	}

	PR5 = z_period-1;	//set the period register

	_T5IF = 0;	//clear the interrupt flag
	TMR5  = 0;	//Clear the timer to zero
	_T5IE = 1;	//enable the T5 interrupt

}


