//***********************************************
//****************  Stepper.h  ******************
//***********************************************
//
//
//***********************************************
//****************  Stepper.h  ******************
//***********************************************

#include <p24fj128ga010.h>
#include <math.h>
#include "LCD.h"	//to use LCD in this module
#include <stdio.h>
#include <stdlib.h>

extern int GX_On;
extern int GY_On;
extern int GZ_On;
extern int GI_On;
extern int GJ_On;

extern float GX;
extern float GY;
extern float GZ;
extern float GI_Val;
extern float GJ_Val;
extern int F_Code;
extern int M_Code;

#define INPUT	1
#define OUTPUT	0

//Lead screw and motor details
#define ScrewPitch_mm			10
#define Step_Rev				200
#define StepDiv2_Rev			400
#define StepDiv4_Rev			800
#define StepDiv8_Rev			1600

//Pulley 20T x 2.5mmpitch
//1 rev = 50mm
//Steps / rev at 16th = 3200
//Steps / rev at 8th = 1600
#define StepsPer_mm				64 // 64 was 16th
#define StepsPer_Inch			100 // 64 was 100
#define ZStepsPermm             1066

//  Axis definitions
#define X_Step		PORTAbits.RA0
#define X_Dir		PORTAbits.RA1
#define Y_Step		PORTAbits.RA2
#define Y_Dir		PORTAbits.RA3
#define Z_Step		PORTAbits.RA4
#define Z_Dir		PORTAbits.RA5
#define A_Step		PORTAbits.RA6
#define A_Dir		PORTAbits.RA7
#define Limit		PORTAbits.RA8

//	Feed rates defined as timer1 ticks ie frequency of pulse
// in/min
//#define Feed_100	1	//fast
//#define Feed_90		2
//#define Feed_80		3
//#define Feed_70		4
//#define Feed_60		5
//#define Feed_50		6
//#define Feed_40		7
//#define Feed_30		8
//#define Feed_20		9
//#define Feed_10		10 //slow

/*****************************************************************************/
/*                                 Functions                                */
/*****************************************************************************/

//External Functions
void Stepper_InitIO(void);
void Stepper_LoadNewValues(int Feed_Code, int Machine_Code);
void Stepper_CalcStepsLine(int Units);
void Stepper_CalcStepsArc(int Units);
void Stepper_Run3DLine(void);
void Stepper_RunArcLine(int Direction);
void Stepper_SaveValues(void);

/*****************************************************************************/

//Internal Functions
void Calc_Delta_Values(int SX_On, int SY_On, int SZ_On,
			 int SI_On, int SJ_On);
void SetXDir(signed long x1,signed long x2);
void SetYDir(signed long y1,signed long y2);
void SetZDir(signed long z1,signed long z2);

void SingleStep(int xs, int ys, int zs);
void SingleStepInt(int xs, int ys, int zs);
void SingleStepX(long x_period,int dir);
void SingleStepY(long y_period,int dir);
void SingleStepZ(long z_period,int dir);

long f_to_ul(float fNumber); //float to unsigned long
signed long f_to_sl(float fNumber); //float to signed long

void Bresenham_2D_Line(int x0, int y0, int x1, int y1);
void Bresenham_3D_Line(signed int x1, signed int y1, signed int z1,
						 signed int x2, signed int y2, signed int z2);

void B_Arc_CCW_Oct1357(signed int x_mid, signed int y_mid,
						 signed int r, int Octant);
void B_Arc_CCW_Oct2468(signed int x_mid, signed int y_mid,
						 signed int r, int Octant);
void B_Arc_CW_Oct1357(signed int x_mid, signed int y_mid,
						 signed int r, int Octant);
void B_Arc_CW_Oct2468(signed int x_mid, signed int y_mid,
						 signed int r, int Octant);
void Move_To_Start(signed long CentreX,signed long CentreY);

int Get_Start_Octant(signed int x, signed int y, int G2orG3);
int Get_End_Octant(signed int x, signed int y, int G2orG3);

void RunOctant(signed int xmid, signed int ymid,
						 signed int r, int Octant, int Direction);



